#!/usr/bin/env python3
# @author yutent<yutent.io@gmail.com>
# @date 2024/01/15 15:38:34

import subprocess, json, re
from datetime import datetime

def toISOTime(time_str):
  _time = datetime.strptime(time_str, '%Y-%m-%d %H:%M:%S %z %Z')
  return _time.isoformat()



class Docker:

  def containers(self, all = True):
    if all:
      cmd = 'docker container ls -a --format=json'
    else:
      cmd = 'docker container ls --format=json'

    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)

    out = out.stdout.decode().strip().split("\n")
    out = [json.loads(it) for it in out]

    return [{
      "id": it['ID'], 
      "name": it['Names'], 
      "image": it['Image'], 
      "cmd":it['Command'],
      "state": it['State'],
      'status': it['Status'],
      'port': it['Ports'].split(', ')[0],
      'created': toISOTime(it['CreatedAt']),
      'last': it['RunningFor'],
    } for it in out]


  def images(self):
    cmd = 'docker images --format=json'

    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)

    out = out.stdout.decode().strip().split("\n")
    out = [json.loads(it) for it in out]

    return [{
      "id": it['ID'], 
      "name": it['Repository'], 
      "tag": it['Tag'], 
      'size': it['Size'],
      'created': toISOTime(it['CreatedAt']),
    } for it in out]
  

  def services(self):
    cmd = 'docker compose ls --format=json'

    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)

    out = json.loads(out.stdout.decode().strip())

    return [{
      "name": it['Name'], 
      'config': it['ConfigFiles'],
      'status': it['Status'],
    } for it in out]
  

  def networks(self):
    cmd = 'docker network ls --format=json'

    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)

    out = out.stdout.decode().strip().split("\n")
    out = [json.loads(it) for it in out]

    # return out
    return [{
      "id": it['ID'], 
      "name": it['Name'], 
      "driver": it['Driver'], 
      'scope': it['Scope'],
      'ipv6': it['IPv6'],
      'labels': it['Labels'],
      'internal': it['Internal'],
      'created': toISOTime(re.sub(r'\.\d+', '', it['CreatedAt'])),
    } for it in out]


  def volumes(self):
    cmd = 'docker volume ls --format=json'

    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)

    out = out.stdout.decode().strip().split("\n")
    out = [json.loads(it) for it in out]

    return [{
      "name": it['Name'], 
      "driver": it['Driver'], 
      'scope': it['Scope'],
      'availability': it['Availability'],
      'mountpoint': it['Mountpoint'],
      'size': it['Size'],
      'status': it['Status'],
    } for it in out]


  def stop_service(self, config = ''):
    cmd = 'docker compose -f ' + config + ' stop'
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)

    return out.stderr.decode().strip()
  

  def prune_volumes(self):
    cmd = 'docker volume prune -a -f'
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)

    return out.stderr.decode().strip()
  

  def rm_volume(self, name = ''):
    cmd = 'docker volume rm ' + name
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)

    return out.stderr.decode().strip()
  

  def prune_networks(self):
    cmd = 'docker network prune -f'
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)

    return out.stderr.decode().strip()
  

  def rm_network(self, id = ''):
    cmd = 'docker network rm ' + id
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)

    return out.stderr.decode().strip()
  

  def start(self, id = ''):
    cmd = 'docker start ' + id
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)

    return out.stderr.decode().strip()
  


  def stop(self, id = ''):
    cmd = 'docker stop ' + id
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)

    return out.stderr.decode().strip()


  def rm(self, id = ''):
    cmd = 'docker rm ' + id
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    return out.stderr.decode().strip()


  def rmi(self, id = ''):
    cmd = 'docker rmi ' + id
    out = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    return out.stderr.decode().strip()